@@ -4,6 +4,8 @@
 import datetime
 import requests
 
+from urllib.parse import urlencode
+
 
 class GitHub(object):
     """GitHub class."""
@@ -15,6 +17,7 @@ class GitHub(object):
             owner_team=None,
             role_team=None,
             verbose=False,
+            debug_user=None,
     ):
         """Initialize an GitHub class instance."""
         # set github token
@@ -39,35 +42,53 @@ class GitHub(object):
         # set the headers for authorized requests
         self.headers = {'Authorization': 'token {}'.format(self.token)}
 
+        # set a debug user dict that can be used by the App class
+        self.debug_user = debug_user
+
     #
     # Sub Classes
     #
-    def client(self, auth):
+    def app(self, auth=None):
+        """Return an App instance."""
+        from bits.github.app import App
+        return App(auth, self, debug_user=self.debug_user)
+
+    def client(self, auth=None):
         """Return a Client instance."""
         from bits.github.client import Client
         return Client(auth, self)
 
-    def datastore(self, auth):
+    def datastore(self, auth=None):
         """Return a Datastore instance."""
         from bits.github.datastore import Datastore
         return Datastore(auth, self)
 
-    def firestore(self, auth):
+    def firestore(
+        self,
+        auth=None,
+        project='broad-bitsdb-firestore',
+        bitsdb_project='broad-bitsdb-prod',
+    ):
         """Return a Firestore instance."""
         from bits.github.firestore import Firestore
-        return Firestore(auth, self)
+        return Firestore(
+            auth,
+            self,
+            project=project,
+            bitsdb_project=bitsdb_project,
+        )
 
-    def helpers(self, auth):
+    def helpers(self, auth=None):
         """Return a Firestore instance."""
         from bits.github.helpers import Helpers
         return Helpers(auth, self)
 
-    def sync(self, auth):
+    def sync(self, auth=None):
         """Return an Update instance."""
         from bits.github.sync import Sync
         return Sync(auth, self)
 
-    def update(self, auth):
+    def update(self, auth=None):
         """Return an Update instance."""
         from bits.github.update import Update
         return Update(auth, self)
@@ -75,7 +96,7 @@ class GitHub(object):
     #
     # Helpers
     #
-    def get(self, url, headers={}, params={}):
+    def get(self, url, headers={}, params={}, raise_for_status=True):
         """Return a response from a GET call."""
         # add any additional headers
         headers = {**self.headers, **headers}
@@ -84,7 +105,8 @@ class GitHub(object):
         response = requests.get(url, headers=headers, params=params)
 
         # raise for status
-        response.raise_for_status()
+        if raise_for_status:
+            response.raise_for_status()
 
         # return json
         return response
@@ -146,6 +168,42 @@ class GitHub(object):
 
         return items_list
 
+    #
+    # Login
+    #
+    def get_oauth_authorize_url(self, client_id, redirect_uri, scopes, state):
+        """Return the oauth2 URL for linking one's GitHub account."""
+        base_url = 'http://github.com/login/oauth/authorize'
+        params = {
+            'client_id': client_id,
+            'redirect_uri': redirect_uri,
+            'scopes': scopes,
+            'state': state,
+        }
+        return '{}?{}'.format(base_url, urlencode(params))
+
+    def get_oauth_access_token(self, client_id, client_secret, redirect_uri, code, state):
+        """Return an access token for a user."""
+        # POST /login/oauth/access_token
+        url = 'https://github.com/login/oauth/access_token'
+        headers = {'Accept': 'application/json'}
+        params = {
+            'client_id': client_id,
+            'client_secret': client_secret,
+            'code': code,
+            'redirect_uri': '{}/'.format(redirect_uri),
+            'state': state,
+        }
+
+        # get response
+        response = requests.post(url, headers=headers, params=params)
+
+        # raise for status
+        response.raise_for_status()
+
+        # return access token
+        return response.json().get('access_token')
+
     #
     # Organizations
     #
@@ -192,6 +250,25 @@ class GitHub(object):
     #
     # Org Members
     #
+    def check_org_membership(self, member):
+        """Return a user's organization membership."""
+        # GET /orgs/:org/members/:username
+        url = '{}/orgs/{}/members/{}'.format(self.base_url, self.org, member)
+        response = self.get(url, raise_for_status=False).json()
+
+        # requester is an organization member and user is a member
+        if response.status_code == 204:
+            return True
+
+        # requester is an organization member and user is not a member
+        if response.status_code == 404:
+            return False
+
+        # raise for status
+        response.raise_for_status
+
+        return False
+
     def get_org_members(self, filterString=None, role=None, insecure=False):
         """Return a list of organization members."""
         # GET /orgs/:org/members
@@ -227,7 +304,7 @@ class GitHub(object):
         """Delete an invitation to the organization."""
         # DELETE /orgs/:org/memberships/:username
         url = '{}/memberships/{}'.format(self.org_base_url, member)
-        return requests.put(url, headers=self.headers).json()
+        return requests.delete(url, headers=self.headers)
 
     #
     # Org Outside Collaborators
@@ -235,19 +312,15 @@ class GitHub(object):
     def get_org_outside_collaborators(self, filterString=None, insecure=False):
         """Return a list of organization outside collaborators."""
         # GET /orgs/:org/outside_collaborators
-        headers = {
-            # 'Accept': 'application/vnd.github.korra-preview'
-        }
-        params = {
-            'filter': filterString,
-        }
+        # headers = {'Accept': 'application/vnd.github.korra-preview'}
+        params = {'filter': filterString}
         if insecure:
             params['filter'] = '2fa_disabled'
-        return self.get_list(self.org_base_url, 'outside_collaborators', headers=headers, params=params)
+        return self.get_list(self.org_base_url, 'outside_collaborators', params=params)
 
     def get_org_outside_collaborators_dict(self, filterString=None, insecure=False, key='id'):
         """Return a dict of organization outside collaborators."""
-        return self.get_dict(self.get_org_members(filterString, insecure), key)
+        return self.get_dict(self.get_org_outside_collaborators(filterString, insecure), key)
 
     #
     # Org Public Members
