@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """Datastore Class file."""
 
 from google.cloud import datastore
@@ -6,14 +7,29 @@ from google.cloud import datastore
 class Datastore(object):
     """Datastore class."""
 
-    def __init__(self, auth, github, project='broad-github'):
+    def __init__(
+        self,
+        auth=None,
+        github=None,
+        project='broad-github'
+    ):
         """Initialize a class instance."""
+        self.auth = auth
         self.datastore = datastore
         self.github = github
         self.project = project
 
+        self.verbose = False
+        if self.github:
+            self.verbose = self.github.verbose
+
         self.db = datastore.Client(self.project)
 
+    def delete_user(self, github_id):
+        """Delete a single user from Datastore."""
+        key = self.db.key('GitHubUser', github_id)
+        self.db.delete(key)
+
     def list_entities(self, kind):
         """Return a list of all entities of a given kind."""
         query = self.db.query(kind=kind)
