@@ -7,16 +7,28 @@ from google.cloud import firestore
 class Firestore(object):
     """Firestore class."""
 
-    def __init__(self, auth, github, project='broad-bitsdb-firestore'):
+    def __init__(
+        self,
+        auth=None,
+        github=None,
+        project='broad-bitsdb-firestore',
+        bitsdb_project='broad-bitsdb-prod',
+    ):
         """Initialize a class instance."""
+        self.auth = auth
+        self.firestore = firestore
         self.github = github
         self.project = project
+        self.bitsdb_project = bitsdb_project
 
-        self.firestore = firestore
-        self.db = firestore.Client(self.project)
+        self.verbose = False
+        if self.github:
+            self.verbose = self.github.verbose
 
-        self.bitsdb_project = 'broad-bitsdb-prod'
-        self.bitsdb = firestore.Client(self.bitsdb_project)
+        # set the default db object
+        self.db = firestore.Client(project=self.project)
+        self.app = firestore.Client()
+        self.bitsdb = firestore.Client(project=self.bitsdb_project)
 
     def get_dict(self, items, key='id'):
         """Return a dict from a list."""
@@ -28,6 +40,13 @@ class Firestore(object):
             data[k] = i
         return data
 
+    #
+    # Collections from Firestore data warehouse: broad-bitsdb-firestore
+    #
+
+    #
+    # github members
+    #
     def get_members(self):
         """Return a list of members."""
         query = self.db.collection('github_members')
@@ -40,28 +59,9 @@ class Firestore(object):
         """Return a dict of members by member id."""
         return self.get_dict(self.get_members(), key)
 
-    def get_people(self):
-        """Return a list of people."""
-        fields = [
-            'email_username',
-            'emplid',
-            'full_name',
-            'github_id',
-            'github_login',
-            'google_id',
-            'terminated',
-            'username',
-        ]
-        query = self.db.collection('people_people').select(fields)
-        people = []
-        for item in query.stream():
-            people.append(item.to_dict())
-        return people
-
-    def get_people_dict(self, key='email_username'):
-        """Return a dict of people by username."""
-        return self.get_dict(self.get_people(), key)
-
+    #
+    # github repos
+    #
     def get_repos(self):
         """Return a list of repos."""
         query = self.db.collection('github_repos')
@@ -74,6 +74,9 @@ class Firestore(object):
         """Return a dict of repos by repo id."""
         return self.get_dict(self.get_repos(), key)
 
+    #
+    # github repos with outside collaborators
+    #
     def get_repos_outside_collaborators(self):
         """Return a list of repos outside collaborators."""
         query = self.db.collection('github_repos_outside_collaborators')
@@ -86,6 +89,9 @@ class Firestore(object):
         """Return a dict of repos outside collaborators."""
         return self.get_dict(self.get_repos_outside_collaborators(), key)
 
+    #
+    # github teams
+    #
     def get_teams(self):
         """Return a list of teams."""
         query = self.db.collection('github_teams')
@@ -98,18 +104,6 @@ class Firestore(object):
         """Return a dict of teams by team id."""
         return self.get_dict(self.get_teams(), key)
 
-    def get_team_syncs(self):
-        """Return a list of team syncs."""
-        query = self.bitsdb.collection('github_team_syncs')
-        team_syncs = []
-        for item in query.stream():
-            team_syncs.append(item.to_dict())
-        return team_syncs
-
-    def get_team_syncs_dict(self, key='github_team_slug'):
-        """Return a dict of team syncs by team id."""
-        return self.get_dict(self.get_team_syncs(), key)
-
     def get_teams_members(self):
         """Return a list of team syncs."""
         query = self.db.collection('github_teams_members')
@@ -133,3 +127,109 @@ class Firestore(object):
     def get_teams_repos_dict(self, key='id'):
         """Return a dict of team syncs by team id."""
         return self.get_dict(self.get_teams_repos_dict(), key)
+
+    #
+    # people
+    #
+    def get_people(self):
+        """Return a list of people."""
+        fields = [
+            'email_username',
+            'emplid',
+            'full_name',
+            'github_id',
+            'github_login',
+            'google_id',
+            'terminated',
+            'username',
+        ]
+        query = self.db.collection('people_people').select(fields)
+        people = []
+        for item in query.stream():
+            people.append(item.to_dict())
+        return people
+
+    def get_people_dict(self, key='email_username'):
+        """Return a dict of people by username."""
+        return self.get_dict(self.get_people(), key)
+
+    #
+    # Collections from Firestore Bitsdb: broad-bitsdb-prod
+    #
+
+    #
+    # github team syncs
+    #
+    def get_team_syncs(self):
+        """Return a list of team syncs."""
+        query = self.bitsdb.collection('github_team_syncs')
+        team_syncs = []
+        for item in query.stream():
+            team_syncs.append(item.to_dict())
+        return team_syncs
+
+    def get_team_syncs_dict(self, key='github_team_slug'):
+        """Return a dict of team syncs by team id."""
+        return self.get_dict(self.get_team_syncs(), key)
+
+    def get_user_team_syncs(self, google_id):
+        """Return a list of team syncs for the given user."""
+        fields = [
+            'github_team',
+            'github_team_slug',
+            'google_group',
+        ]
+        groups_ref = self.bitsdb.collection('github_group_members')
+        query = groups_ref.where(u'members', u'array_contains', google_id).select(fields)
+        teams = []
+        for doc in query.stream():
+            sync = doc.to_dict()
+            teams.append(sync)
+        return teams
+
+    #
+    # Collections from GitHub App Firetore: broad-github-app
+    #
+    def delete_stored_token(self, google_id):
+        """Delete a token stored in firestore."""
+        return self.app.collection('tokens').document(google_id).delete()
+
+    def delete_stored_github_user(self, github_id):
+        """Delete a github user stored in firestore."""
+        return self.app.collection('github_users').document(str(github_id)).delete()
+
+    def get_config(self):
+        """Return the config settings from Firestore."""
+        config = {}
+        for doc in self.app.collection('settings').stream():
+            config[doc.id] = doc.to_dict()
+        return config
+
+    def get_stored_github_user(self, github_id):
+        """Return the stored user for the given github user id."""
+        return self.app.collection('github_users').document(str(github_id)).get().to_dict()
+
+    def get_stored_token(self, google_id):
+        """Return the stored token for the given google user id."""
+        return self.app.collection('tokens').document(google_id).get().to_dict()
+
+    def get_tokens(self):
+        """Return all token records from firestore."""
+        tokens = []
+        for doc in self.app.collection('tokens').stream():
+            tokens.append(doc.to_dict())
+        return tokens
+
+    def get_user(self, google_id):
+        """Return the stored token for the given google user id."""
+        return self.app.collection('users').document(google_id).get().to_dict()
+
+    def get_users(self):
+        """Return a list of users."""
+        users = []
+        ref = self.app.collection('users')
+        for doc in ref.stream():
+            u = doc.to_dict()
+            u['id'] = doc.id
+            users.append(u)
+        return users
